/* Linux Prism II Stumbler - Utility Scan for 802_11 networks under Linux
 * 
 * Project : WifiScanner (c) 2002 Herv Schauer Consultants
 * Usage : This utility is written for use with IEEE 802.11 adapters based
 * on Intersil's PRISM II chipset (PCMCIA).
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: pcap.c,v 1.3 2005/11/03 15:28:53 poggij Exp $
 */

#include <include.h>
#include <src/pcap_file.h>
#include <src/wlan-ng.h>
#include <src/functions.h>
#include <src/crt_io.h>

static char *ID = "$Id: pcap.c,v 1.3 2005/11/03 15:28:53 poggij Exp $";

// All extern value you want
//extern unsigned int DebugLevel;
//extern UINT8 SingleChannel;
//extern UINT8 TypeOfCard;
extern ConfigStruct config;
extern p80211_caphdr_t wlan_header;
extern WINDOW *Sum_WND, *RealTime_WND;

static CaptureArg ca;
static char errbuf[PCAP_ERRBUF_SIZE];

#define STR_MAX 80

//-------------

// Get packet from card
int getPacketPCAP(UINT8 * buf, int maxlen)
{
  struct pcap_pkthdr pktHdr;
  u_char *ret;
  fd_set rs;

  FD_ZERO(&rs);
  FD_SET(0, &rs);

  warning("before\n");
  ret = (u_char *) pcap_next(ca.pcap, &pktHdr);
  warning("after\n");

  // If no problem and packet is enought big (with data)
  if ((ret) && (pktHdr.len >= 1)) {
#if 1
    // Function to dump in hexa the message send or sent to the driver
    DumpHexPaquets(RealTime_WND, ret, 0x40);
#endif
    memcpy_buff(buf, ret + ca.offset, pktHdr.len + 0x00);
    // Fill Header
    // TODO : find this information in any maner ?!
    wlan_header.version = 0;    // It's a reduced capture frame format
    wlan_header.length = 0;     // Not used for now
    wlan_header.mactime = 0;
    wlan_header.hosttime = 0;
    wlan_header.phytype = 0;    // Not used for now
    wlan_header.channel = 0;
    wlan_header.datarate = 0;   // datarate is in units of 100kbps.
    wlan_header.antenna = 0;    // Not used for now
    wlan_header.priority = 0;   // Not used for now
    wlan_header.ssi_type = 0;   // Not used for now
    wlan_header.ssi_signal = 0;
    wlan_header.ssi_noise = 0;
    wlan_header.preamble = 0;   // Not used for now
    wlan_header.encoding = 0;   // Not used for now
    return pktHdr.len;
  } else {
    return (0);                 // Noting to read
  }
}

int openPacketPCAP(char *pcap_name)
{
  int DataLink;

  ca.pcap = pcap_open_offline(pcap_name, errbuf);
  if (ca.pcap) {
    DataLink = pcap_datalink(ca.pcap);
    switch (DataLink) {
    case DLT_PRISM_HEADER:
      debug(2, "pcap_datalink(ca.pcap) = %d = DLT_PRISM_HEADER\n",
            DataLink);
      ca.offset = 0x90;
      break;
    case DLT_IEEE802_11:
      debug(2, "pcap_datalink(ca.pcap) = %d = DLT_IEEE802_11\n", DataLink);
      ca.offset = 0;
      break;
    case DLT_AIRONET_HEADER:
      debug(2, "pcap_datalink(ca.pcap) = %d = DLT_AIRONET_HEADER:\n",
            DataLink);
      ca.offset = 0;
      break;
    case DLT_LINUX_SLL:        //113
      debug(2, "pcap_datalink(ca.pcap) = %d = DLT_LINUX_SLL:\n", DataLink);
      ca.offset = 16;
      break;
    default:                   //COOKED
      debug(2, "pcap_datalink(ca.pcap) = %d = Default to COOKED:\n",
            DataLink);
      ca.offset = 16;
    }
    return 1;
  }
  return -1;
}

void closePacketPCAP(void)
{
  pcap_close(ca.pcap);
}
